// carriers
// carry your teammates

globalvar carriers;
carriers = id;

// "Required plugins should automatically set themselves to required"
global.serverPluginsRequired = 1;

// CONSTS
//
//
//		ACTIONS
GRAB			= 0;
RELEASE			= 3;
//		SYNC
PLAYER_SYNC		= 4;
CHAR_SYNC		= 5;
CONSTS_UPDATE	= 6;
WELCOME			= 7;
//		MODE when you carry:
//			you can either use your weapon and be slow,
//			or run faster and not be able to
MODE_RUN		= 1;
MODE_SHOOT		= 2;
//		ENABLE or disable
//			you can prevent players from carrying you
CARRY_ENABLE	= 8;
CARRY_DISABLE	= 9;
//		FORBID or allow
//			you can prevent one player in particular from carrying you
FORBID			= 11;
ALLOW			= 12;
//		BAN or unban
//			host can prevent one player in particular from carrying anyone
BAN				= 13;
UNBAN			= 14;




// PERSISTENT VARS
//
//
ini_open("gg2.ini");

RUN_SLOWDOWN_PER_PLAYER		= ini_read_real("Plugins", "carriers_RUN_SLOWDOWN_PER_PLAYER", 0.04);
SHOOT_SLOWDOWN_PER_PLAYER	= ini_read_real("Plugins", "carriers_SHOOT_SLOWDOWN_PER_PLAYER", 0.04);
RUN_SLOWDOWN				= ini_read_real("Plugins", "carriers_RUN_SLOWDOWN", 1);
INTEL_SLOWDOWN				= ini_read_real("Plugins", "carriers_INTEL_SLOWDOWN", 0.7);
SHOOT_SLOWDOWN				= ini_read_real("Plugins", "carriers_SHOOT_SLOWDOWN", 0.82);
MIN_SPEED					= ini_read_real("Plugins", "carriers_MIN_SPEED", 0.2);
STACK_MAX_HEIGHT			= ini_read_real("Plugins", "carriers_STACK_MAX_HEIGHT", 255);
HEAVIER_HP					= ini_read_real("Plugins", "carriers_HEAVIER_HP", false); // are you slower when you carry classes with more hp

grabKey						= ini_read_real("Plugins", "carriers_grabKey", ord("Q"));
modeKey						= ini_read_real("Plugins", "carriers_modeKey", ord("V"));
carryEnable					= ini_read_real("Plugins", "carriers_enable", true);

ini_close();





// SCRIPT
//
//

// PLAYER ATTRIBUTES
//
object_event_add(Player, ev_create, 0, '
	carryWelcomed = false;
	carryUpdated = false;
	carryEnable	= true;
	carryForbidden = false;
	forbiddenCarriers = ds_list_create();
');
object_event_add(Player, ev_step, ev_step_begin, '
	if(not variable_local_exists("carryEnable")) {
		carryWelcomed = false;
		carryUpdated = false;
		carryEnable	= true;
		carryForbidden = false;
		forbiddenCarriers = ds_list_create();
	}
');
object_event_add(Player, ev_destroy, 0, '
	var index;
	with(Player) {
		if(id != other.id) {
			index = ds_list_find_index(forbiddenCarriers, other.id)
			if(index > -1) {
				ds_list_delete(forbiddenCarriers, index);
			}
		}
	}
	ds_list_destroy(forbiddenCarriers);
');


// CHARACTER ATTRIBUTES
//
object_event_add(Character, ev_create, 0, '
	carrier				= noone;
	carry				= noone;
	carryRunning		= false;
	originalRunPower	= runPower;
');
object_event_add(Character, ev_step, ev_step_begin, '
	if(not variable_local_exists("carrier")) {
		carrier				= noone;
		carry				= noone;
		originalRunPower	= runPower;
		carryRunning		= false;
	}
');
object_event_add(Character, ev_destroy, 0, '
	with(carrier) {
		carry = noone;
	}
	with(carry) {
		x = other.x;
		if(other.player.class == CLASS_QUOTE) {
			y = other.y - 12;
		} else {
			y = other.y;
		}
		carrier = noone;
	}
	carrier	= noone;
	carry	= noone;
');


//	//	//	"PHYSICS" RUN HERE
//
//
object_event_add(Character, ev_step, ev_step_begin, '
	currentWeapon.visible = true;
	if(carrier == noone) {
		
		// run power
		runPower = originalRunPower;
		if(player.class == CLASS_SNIPER)
		if(zoomed)
			runPower = 0.6;
		if(player.class == CLASS_HEAVY)
		if(keyState & $10) {
			runPower = 0.3;
		}
		if(player.class == CLASS_SPY)
		if(stabbing)
			runPower = 0;
			
		if(carry != noone) {
			var stackSlowdown;
			
			if(carriers.HEAVIER_HP) {
				stackSlowdown = execute_string(carriers.stackHeight, id) - 1;
			} else {
				stackSlowdown = (execute_string(carriers.stackHealth, id) / maxHp) - 1;
			}
			
			if(carryRunning) {
				runPower *= (carriers.RUN_SLOWDOWN - stackSlowdown * carriers.RUN_SLOWDOWN_PER_PLAYER);
			} else {
				runPower *= (carriers.SHOOT_SLOWDOWN - stackSlowdown * carriers.SHOOT_SLOWDOWN_PER_PLAYER);
			}
			
			if(execute_string(carriers.stackHasIntel, id)) {
				runPower *= carriers.INTEL_SLOWDOWN;
			}
			
			runPower = max(carriers.MIN_SPEED, runPower);
			// gotta redo that because of the if/else above
			if(player.class == CLASS_SPY)
			if(stabbing)
				runPower = 0;
			
			// carrier weapon disabling
			if(carryRunning) {
				with(currentWeapon) {
					readyToShoot = false;
					if(variable_local_exists("refireTime"))
						alarm[0] = refireTime / global.delta_factor;
					var al5;
					al5 = 0;
					if(variable_local_exists("reloadBuffer"))
						al5 += reloadBuffer;
					if(variable_local_exists("reloadTime"))
						al5 += reloadTime;
					al5 /= global.delta_factor;
					alarm[5] = al5;
					visible = false;
				}
			}
		}
	}
');

object_event_add(Character, ev_step, ev_step_end, '
	if(carry != noone) {
		// carried players position
		if(player.class != CLASS_QUOTE) {
			carry.x = floor(x - (8 * image_xscale));
		} else {
			carry.x = floor(x);
		}
		carry.y = floor(y - 18);
		// dont stick them in roof
		with(carry) {
			characterHitObstacle();
		}
		carry.hspeed = hspeed;
		carry.vspeed = vspeed;
	}
');


// COLLISIONS
//
// after testing, FREEZETIME MAKES EVERY SINGLE GATE CHILD 
// OVERRIDE THEIR PARENT GATE COLLIDE EVENT
// so I have to use every single child instead :z7:
collideWithGateScript = '
	if(carry != noone) {
		execute_string(carriers.doRelease, player);
	}
';
object_event_add(Character, ev_collision, Gate, collideWithGateScript);
object_event_add(Character, ev_collision, RedTeamGate, collideWithGateScript);
object_event_add(Character, ev_collision, BlueTeamGate, collideWithGateScript);
object_event_add(Character, ev_collision, RedTeamGate2, collideWithGateScript);
object_event_add(Character, ev_collision, BlueTeamGate2, collideWithGateScript);
object_event_add(Character, ev_collision, IntelGate, collideWithGateScript);
object_event_add(Character, ev_collision, TeamGate, collideWithGateScript);






// EXECUTE OTHER FILES
//
//
execute_file(directory + "\Controls.gml");
execute_file(directory + "\Events.gml");
execute_file(directory + "\Menus.gml");
execute_file(directory + "\Networking.gml");
execute_file(directory + "\NetworkingScripts.gml");
execute_file(directory + "\Notifications.gml");
execute_file(directory + "\Scripts.gml");